#!/usr/bin/python2.7
# Copyright 2011 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Validation routines for discovery elements.

This is a singleton class which provides validation for names which can
appear in discovery documents
"""

__author__ = 'wclarkso@google.com (Will Clarkson)'

import re


# The first character must alpha (a-zA-Z), a slash (/), a '@', or an
# underscore (no single digits).  Subsequent characters can be alpha
# numeric. We also permit them to have a slash(/), underscore (_), dot
# (.) or dash (-).  NOTE: the '$' character is to get around $ref or foo$bar
# variable names in some APIs.
_VARNAME_REGEX = re.compile(
    r'^[a-zA-Z]$|([a-zA-Z_/$@][a-zA-Z0-9_./$-]+)$')

_API_NAME_REGEX = re.compile(r'[a-z][a-zA-Z0-9_]*$')
_API_VERSION_REGEX = re.compile(r'[a-z0-9][a-zA-Z0-9._-]*$')


class ValidationError(ValueError):
  pass


def Validate(name):
  """Validates the name against a regular expression object.

  If the name matches the regular expression, we return nothing.
  If the name fails to match, we generate an exception.

  Args:
    name: (str) name of variable or method
  Returns:
    (str) The name.

  Raises:
    ValidationError: An Error if name does not conform to style
  """
  if _VARNAME_REGEX.match(name) is None:
    raise ValidationError(
        'Variable %s does not conform to style guide' % name)
  return name


def ValidateApiName(api_name):
  """Validates a potential API name.

  An API name must match the regular expression[a-z0-9][a-zA-Z0-9_]*

  Args:
    api_name: (str) The API name to check.
  Returns:
    (str) The api name.

  Raises:
    ValidationError: An Error if name does not conform to style
  """
  if _API_NAME_REGEX.match(api_name) is None:
    raise ValidationError(
        'API name %s does not conform to style guide' % api_name)
  return api_name


def ValidateApiVersion(api_version):
  """Validates a potential API version.

  An API version must match the regular expression[a-z0-9][a-zA-Z0-9.]*

  Args:
    api_version: (str) The API version to check.
  Returns:
    (str) The api version.

  Raises:
    ValidationError: An Error if version does not conform to style
  """
  if _API_VERSION_REGEX.match(api_version) is None:
    raise ValidationError(
        'API version %s does not conform to style guide' % api_version)
  return api_version


def ValidateAndSanitizeComment(comment_string):
  """Validates a comment string.

  Remove comment terminators (e.g. */) from a string.

  TODO(user): Make a per-language validator. Allow non-dangerous symbols
  depending on language. e.g. */ is OK for Python but not PHP

  Args:
    comment_string: (str|unicode) input comment string
  Returns:
    (unicode) String with invalid character sequences removed
  """
  # Strip anything which is known to be a comment terminator in any
  # supported language.
  invalid_strings = [u'/*',    # C-style Multi-line start
                     u'*/',    # C-style Multi-line end
                     u'\"""',  # Python Multiline string
                     u'///',   # Escaped comment begin
                     u'\\*',   # Escaped Multiline begin
                    ]

  if isinstance(comment_string, str):
    comment_string = comment_string.decode('utf-8')
  change_made = True
  while change_made:
    change_made = False
    # Save original length for easy comparision later
    beginning_length = len(comment_string)

    for substring in invalid_strings:
      # Replace all instances of substring with empty string
      comment_string = comment_string.replace(substring, '')

    # If the length of the string changed, then a replacement occurred.
    # We need to repeat the process until no changes occur
    if len(comment_string) != beginning_length:
      change_made = True

  return comment_string



#!/usr/bin/python2.7
# Copyright 2011 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for cpp_import_manager."""

__author__ = 'ewiseblatt@google.com (Eric Wiseblatt)'

from google.apputils import basetest
from googleapis.codegen.cpp_import_manager import CppImportManager


class MockSchema(object):
  """Mock schema used in place of real schema objects."""

  def __init__(self):
    self._template_values_dict = {}

  def SetTemplateValue(self, template_name, template_value):
    self._template_values_dict[template_name] = template_value

  def GetTemplateValue(self, template_name):
    return self._template_values_dict.get(template_name)


class CppImportManagerTest(basetest.TestCase):

  def setUp(self):
    super(CppImportManagerTest, self).setUp()
    self.mock_schema = MockSchema()
    self.import_manager = CppImportManager(self.mock_schema)

  def testAddImportAndCommit(self):
    # Add a com.google import.
    com_google_import = '"base/integral_types.h"'
    self.assertTrue(self.import_manager.AddImport(com_google_import))
    self.assertFalse(self.import_manager.platform_imports)
    self.assertFalse(self.import_manager.other_imports)
    self.assertTrue(self.import_manager.google_imports)

    # There are no platform imports for C++
    platform_import = '<string>'
    self.assertTrue(self.import_manager.AddImport(platform_import))
    self.assertTrue(self.import_manager.platform_imports)

    # Add a random thing
    other_import = '"Module.h"'
    self.import_manager.AddImport(other_import)

    # Assert the contents of google, other and java imports.
    expected_google_import_set = set()
    expected_google_import_set.add(com_google_import)
    sorted_expected_google_import_set = sorted(expected_google_import_set)
    self.assertEquals(sorted_expected_google_import_set,
                      list(self.import_manager.google_imports))
    self.assertEquals([other_import],
                      list(self.import_manager.other_imports))
    self.assertEquals([platform_import],
                      list(self.import_manager.platform_imports))

    # Assert the contents of class_name_to_qualified_name map.
    self.assertEquals(
        com_google_import,
        self.import_manager._class_name_to_qualified_name[com_google_import])

    # Assert that commit import works.
    # The import_manager combines the platform and google imports together
    # but each segment is first sorted.
    expected_import_list = [
        sorted([platform_import]) + sorted_expected_google_import_set,
        [other_import]]
    self.assertEquals(
        expected_import_list,
        self.mock_schema.GetTemplateValue('importManager').ImportLists())

if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7


from google.apputils import basetest
from googleapis.codegen import java_generator
from googleapis.codegen.utilities import maven_utils


class MavenUtilsTest(basetest.TestCase):

  def testGetMavenArtifactId(self):
    args = [
        ({'api_name': 'adsense'}, 'google-api-services-adsense'),
        ({'api_name': 'adsense', 'owner_domain': 'snorg.com'}, 'adsense'),
        ({'api_name': 'adsense', 'owner_domain': 'snorg.com',
          'package_path': 'snorg.foobar'},
         'snorg-foobar-adsense'),
        ({'api_name': 'admin', 'package_path': 'admin',
          'canonical_name': 'reports'},
         'google-api-services-admin-reports'),
        ]
    for kwargs, artifact_id in args:
      self.assertEquals(artifact_id, maven_utils.GetMavenArtifactId(**kwargs))

  def testMavenArtifactIdAndModulePath(self):

    def TestApi(api_def):
      gen = java_generator.BaseJavaGenerator(api_def,
                                             options={'google_api': True})
      gen.AnnotateApiForLanguage(gen.api)
      api = gen.api

      # drop the leading com
      expected = '-'.join(api.module.path.split('/')[1:])
      actual = maven_utils.GetMavenArtifactId(
          api_def['name'],
          package_path=api_def.get('packagePath'),
          canonical_name=api_def.get('canonicalName'),
          owner_domain=api_def['ownerDomain'])
      self.assertEquals(expected, actual)
    TestApi({'name': 'admin',
             'version': 'directory_v1',
             'canonicalName': 'directory',
             'packagePath': 'admin',
             'ownerDomain': 'google.com'})
    TestApi({'name': 'identitytoolkit',
             'version': 'v1',
             'canonicalName': 'Identity Toolkit',
             'ownerDomain': 'google.com'})
    TestApi({'name': 'identitytoolkit',
             'version': 'v1',
             'packagePath': 'zomby',
             'canonicalName': 'Identity Toolkit',
             'ownerDomain': 'google.com'})

  def testGetMavenGroupId(self):
    self.assertEquals('com.foo.bar', maven_utils.GetMavenGroupId('bar.foo.com'))
    self.assertEquals('com.google.apis',
                      maven_utils.GetMavenGroupId('google.com'))

  def testGetMavenVersion(self):
    google_api = {'name': 'api',
                  'version': 'v1',
                  'revision': 3,
                  'ownerDomain': 'google.com'}
    lv = '1.5'
    self.assertEquals('v1-rev3-1.5',
                      maven_utils.GetMavenVersion(google_api, lv))
    endpoints_api = {'name': 'api',
                     'version': 'v1',
                     'ownerDomain': 'bestbuy.com'}
    self.assertEquals('v1-1.5-SNAPSHOT',
                      maven_utils.GetMavenVersion(endpoints_api, lv))


if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
# Copyright 2013 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Java protobuf library generator.
"""

__author__ = 'yanivi@google.com (Yaniv Inbar)'

from googleapis.codegen import api_library_generator
from googleapis.codegen import data_types
from googleapis.codegen.java_generator import JavaApi
from googleapis.codegen.java_generator import JavaLanguageModel


class JavaProtoGenerator(api_library_generator.ApiLibraryGenerator):
  """Java protobuf code generator."""

  def __init__(self, discovery, language='javaproto', language_model=None,
               options=None):
    if not language_model:
      language_model = self._GetDefaultLanguageModel(options)
    super(JavaProtoGenerator, self).__init__(JavaApi, discovery, language,
                                             language_model, options=options)

  @classmethod
  def _GetDefaultLanguageModel(cls, options=None):
    return JavaLanguageModel(options=options)

  def AnnotateMethod(self, unused_the_api, method, unused_resource):
    """Annotate a Method with Java Proto specific elements.

    Args:
      unused_the_api: (Api) The API tree which owns this method.
      method: (Method) The method to annotate.
      unused_resource: (Resource) The resource which owns this method.

    Raises:
      ValueError: if missing externalTypeName
    """
    for attr in ('requestType', 'responseType'):
      schema = method.get(attr)
      if schema and not isinstance(schema, data_types.Void):
        name = schema.get('externalTypeName')
        if not name:
          raise ValueError('missing externalTypeName for %s (%s of method %s)'
                           % (schema['id'], attr, method['rpcMethod']))
        java_name = schema.get('javaTypeName')
        proto_name = java_name or 'TO_BE_COMPUTED.' + name[name.rfind('.')+1:]
        schema.SetTemplateValue('protoFullClassName', proto_name)




#!/usr/bin/python2.7
# Copyright 2010 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Generator for an API library.

This module specializes TemplateGenerator for building API libraries.
"""

__author__ = 'aiuto@google.com (Tony Aiuto)'

from googleapis.codegen.generator import TemplateGenerator


class ApiLibraryGenerator(TemplateGenerator):
  """TemplateGenerator specialization which produces an API library."""

  def __init__(self, api_loader, discovery, language, language_model=None,
               options=None):
    """Construct an ApiLibraryGenerator.

    Args:
      api_loader: (Api) Method which can construct an Api from discovery.
      discovery: (dict) A discovery definition.
      language: (str) The target language name. This has no semantic meaning
          other than to specify the template set to use.
      language_model: (LanguageModel) The target language data model.
      options: (dict) Code generator options.
    """
    super(ApiLibraryGenerator, self).__init__(language_model=language_model,
                                              options=options)
    # Load the API definition and an prepare it for generating code.
    # TODO(user): package_path and version_package are really changes to the
    # API data model, rather than generator options. Move them to a distinct
    # parameter, and pass them down to the api_loader.
    options = options or {}
    module_path = options.get('package_path')
    if module_path:
      discovery['modulePath'] = module_path
    if options.get('version_package'):
      discovery['version_module'] = True
    self._api = api_loader(discovery)
    self._language = language

  @property
  def api(self):
    return self._api

  @property
  def model_module(self):
    return self._api.model_module

  def GeneratePackage(self, package_writer, path_replacements=None,
                      extra_defines=None):
    """Generate the entire package of an API library.

    Overrides superclass.

    Args:
      package_writer: (LibraryPackage) output package
      path_replacements: (dict) dict holding elements which should be replaced
        if found in a path. (See generator.WalkTemplateTree for details.)
      extra_defines: (dict) Dictionary of extra definitions to provide to the
        templates
    """
    api = self._api
    self.AnnotateApiForLanguage(api)

    self._BuildPathReplacements(path_replacements)
    self._top_level_defines = {'api': api.values}
    self._top_level_defines.update(extra_defines or {})

    self._GenerateLibrarySource(api, package_writer)
    if self._options.get('include_dependencies'):
      self.WalkTemplateTree('dependencies', self._path_replacements, {},
                            self._top_level_defines, package_writer)

  def _BuildPathReplacements(self, path_replacements):
    """Build the set of path replacements used for template tree walking.

    Augments a default list of path replacements with a caller provided set.
    Side effects: Sets self._path_replacements.

    Args:
      path_replacements: (dict) caller provided dict holding elements which
        should be replaced if found in a path.
    """
    self._path_replacements = {
        '___package___': self._api.module.path,
        '___package_name___': self._api.module.name,
        '___language___': self._language,
        # language version will in practice be set, because it
        # comes from features, but may not be in some unit tests.
        '___language_version___': self.language_version or '',
        }

    for key, value in self.features.iteritems():
      if isinstance(value, (unicode, str)):
        self._path_replacements['___features_%s___' % key] = value

    if not isinstance(self._api.values['revision'], (unicode, str)):
        # Make sure revision is a string
      self._path_replacements['___api_revision___'] = str(
          self._api.values['revision'])

    for key, value in self._api.values.iteritems():
      if isinstance(value, (unicode, str)):
        self._path_replacements['___api_%s___' % key] = value

    self._path_replacements.update(path_replacements or {})

  def _GenerateLibrarySource(self, api, source_package_writer):
    """Default operations to generate the package.

    Do all the default operations for generating a package.
    1. Walk the template tree to generate the source.
    2. Add in per-language additions to the source
    3. Optionally copy in dependencies
    4. (Side effect) Closes the source_package_writer.

    Args:
      api: (Api) The Api instance we are writing a libary for.
      source_package_writer: (LibraryPackage) source output package.
    """
    list_replacements = {
        '___models_': ['model', api.ModelClasses()],
        '___topLevelModels_': ['model', api.TopLevelModelClasses()],
        }
    self.WalkTemplateTree('templates', self._path_replacements,
                          list_replacements,
                          self._top_level_defines, source_package_writer)
    # Call back to the language specific generator to give it a chance to emit
    # special case elements.
    self.GenerateExtraSourceOutput(source_package_writer)

  def GenerateExtraSourceOutput(self, source_package_writer):
    """Extension point for subclasses to add extra data to the output.

    A language generator may provide an implementation of this to emit elements
    which cannot be handled by GenerateLibraryPackage.

    Args:
      source_package_writer: (LibraryPackage) An output package writer.
    """
    pass

  def AnnotateApiForLanguage(self, the_api):
    """Add the language specific annotations to an api.

    Performs all the language specific annotations on an API so it is ready to
    use for generating a library surface. This is essentially an impedance match
    between what is expressed in the API definition and how a language specific
    binding can be expressed using only templates.

    Args:
      the_api: (Api) The API to annotate.
    """
    the_api.VisitAll(lambda o: o.SetLanguageModel(self.language_model))
    the_api.void_type.SetLanguageModel(self.language_model)
    self._AnnotateTree(the_api)

  def _AnnotateTree(self, api):
    """Decorate the API tree with language model specific elements.

    Walks the tree and calls annotators on the Methods and Properties.  This
    may be used to supply language specific transforms to an API between the
    time the API is loaded and before we generate code through the templates.

    Should be called after the API is constructed and before we generate
    any code.

    Args:
      api: (Api) The Api.
    """
    self.AnnotateApi(api)
    for schema in api.all_schemas.values():
      # TODO(user): remove this after completing the transition away from
      # package in all the templates
      schema.SetTemplateValue('package', self.model_module)
      self.AnnotateSchema(api, schema)
      for prop in schema.values.get('properties', []):
        self.AnnotateProperty(api, prop, schema)
    for resource in api.values['resources']:
      self.AnnotateResource(api, resource)
    for method in api.values['methods']:
      self.AnnotateMethod(api, method, None)

  def AnnotateApi(self, api):
    """Extension point for subclasses to annotate the API node itself.

    A language generator may provide an implementation for this.

    Args:
      api: (Api) The Api.
    """
    api.SetTemplateValue('topLevelModels', api.TopLevelModelClasses())
    for parameter in api.values.get('parameters') or []:
      self.AnnotateParameter(api, parameter)

  def AnnotateMethod(self, unused_api, method, unused_resource):
    """Extension point for subclasses to annotate Resources.

    A language generator may provide an implementation for this.

    Args:
      unused_api: (Api) The Api.
      method: (Method) The Method to annotate.
      unused_resource: (Resource) The Resource which owns this Method.
    """
    for parameter in method.parameters:
      self.AnnotateParameter(method, parameter)

  def AnnotateParameter(self, method, parameter):
    """Extension point for subclasses to annotate method Parameters.

    A language generator may provide an implementation for this.

    Args:
      method: (Method) The Method this parameter belongs to.
      parameter: (Parameter) The Parameter to annotate.
    """
    pass

  def AnnotateProperty(self, api, prop, schema=None):
    """Extension point for subclasses to annotate Properties.

    A language generator may provide an implementation for this.

    Args:
      api: (Api) The Api.
      prop: (Property) The Property to annotate.
      schema: (Schema) The Schema this Property belongs to.
    """
    pass

  def AnnotateResource(self, api, resource):
    """Extension point for subclasses to annotate Resources.

    A language generator may provide an implementation for this. The default
    walks the Resources methods and sub-resources to annotate those.

    Args:
      api: (Api) The Api which owns this resource.
      resource: (Resource) The Resource to annotate.
    """
    for method in resource.values['methods']:
      self.AnnotateMethod(api, method, resource)
    for r in resource.values['resources']:
      self.AnnotateResource(api, r)

  def AnnotateSchema(self, api, schema):
    """Extension point for subclasses to annotate Schemas.

    A language generator may provide an implementation for this.

    Args:
      api: (Api) The Api.
      schema: (Schema) The Schema to annotate
    """
    pass


class NullLibraryGenerator(ApiLibraryGenerator):
  """Used to flag a language that doesn't do library generation."""
  pass



#!/usr/bin/python2.7
# Copyright 2011 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Dart library generator.

This module generates Dart code from a Google API discovery documents.
"""

__author__ = 'aiuto@google.com (Tony Aiuto)'

from googleapis.codegen import api
from googleapis.codegen import api_library_generator
from googleapis.codegen import language_model
from googleapis.codegen import utilities


class DartLanguageModel(language_model.LanguageModel):
  """A LanguageModel for Dart."""

  language = 'dart'

  _SCHEMA_TYPE_TO_DART_TYPE = {
      'any': 'core.Object',
      'boolean': 'core.bool',
      'integer': 'core.int',
      'long': 'core.int',
      'number': 'core.double',
      'string': 'core.String',
      'object': 'core.Object',
      }

  _DART_KEYWORDS = [
      'break', 'case', 'catch', 'class', 'const', 'continue', 'default',
      'else', 'extends', 'final', 'finally', 'for', 'in', 'is', 'new', 'null',
      'return', 'static', 'super', 'switch', 'throw', 'try', 'var', 'void',
      'while'
      ]

  # We can not create classes which match a Dart keyword or built in object
  # type.
  RESERVED_CLASS_NAMES = _DART_KEYWORDS + _SCHEMA_TYPE_TO_DART_TYPE.values()

  array_of_policy = language_model.NamingPolicy(
      format_string='core.List<{name}>')
  map_of_policy = language_model.NamingPolicy(
      format_string='core.Map<core.String, {name}>')

  def __init__(self):
    super(DartLanguageModel, self).__init__(class_name_delimiter='')

  def GetCodeTypeFromDictionary(self, def_dict):
    """Gets an element's data type from its JSON definition.

    Overrides the default.

    Args:
      def_dict: (dict) The defintion dictionary for this type
    Returns:
      A name suitable for use as a Dart data type
    """
    json_type = def_dict.get('type', 'String')
    native_type = self._SCHEMA_TYPE_TO_DART_TYPE.get(json_type)
    return native_type

  def ToMemberName(self, s, unused_the_api):
    """CamelCase a wire format name into a suitable Dart variable name."""
    candidate = utilities.CamelCase(s)
    candidate = candidate[0].lower() + candidate[1:]
    while candidate in self.RESERVED_CLASS_NAMES:
      candidate += '_'
    return candidate

DART_LANGUAGE_MODEL = DartLanguageModel()


class DartGenerator(api_library_generator.ApiLibraryGenerator):
  """The Dart code generator."""

  def __init__(self, discovery, options=None):
    super(DartGenerator, self).__init__(
        DartApi,
        discovery,
        language='dart',
        language_model=DART_LANGUAGE_MODEL,
        options=options)

  def AnnotateResource(self, the_api, resource):
    """Override default implementation.

    Prepend the resource class name to each sub-resource since Dart doesn't
    support nested classes. Append 'Resource' to the class name so resources
    are less likely to conflict with models.

    Args:
      the_api: (Api) The API this Resource belongs to.
      resource: (Resource) The Resource to annotate.
    """
    for r in resource.values['resources']:
      r.values['className'] = (resource.values['className'] +
                               r.values['className'])
      self.AnnotateResource(the_api, r)
    resource.values['className'] += 'Resource'

    parent_list = resource.ancestors[1:]
    parent_list.append(resource)
    parent_classes = [p.values.get('className') for p in parent_list]
    resource.SetTemplateValue('contextCodeType', '.'.join(parent_classes))
    super(DartGenerator, self).AnnotateResource(the_api, resource)

  def AnnotateMethod(self, the_api, method, unused_resource):
    """Add Dart-specific annotations and naming schemes."""
    parent_list = method.ancestors[1:]
    parent_list.append(method)
    parent_classes = [p.values.get('className') for p in parent_list]
    method.SetTemplateValue('contextCodeType', '.'.join(parent_classes))
    super(DartGenerator, self).AnnotateMethod(the_api, method, None)


class DartApi(api.Api):
  """An Api with Dart annotations."""

  def __init__(self, discovery_doc, **unused_kwargs):
    super(DartApi, self).__init__(discovery_doc)

  # pylint: disable=unused-argument
  def ToClassName(self, s, element, element_type=None):
    """Convert a discovery name to a suitable Dart class name.

    Overrides the default.

    Args:
      s: (str) A rosy name of data element.
      element: (object) The object we need a class name for.
      element_type: (str) The kind of element (resource|method) to name.
    Returns:
      A name suitable for use as a class in the generator's target language.
    """

    candidate = utilities.CamelCase(s)
    if isinstance(element, api.Api):
      candidate += 'Api'
    while candidate in DartLanguageModel.RESERVED_CLASS_NAMES:
      candidate += '_'
    return candidate



#!/usr/bin/python2.7
# Copyright 2011 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""GWT/Java library generator based on RequestFactory."""

__author__ = 'robertvawter@google.com (Bob Vawter)'

from googleapis.codegen import java_generator
from googleapis.codegen import language_model
from googleapis.codegen import utilities
from googleapis.codegen.import_definition import ImportDefinition


class GwtGenerator(java_generator.BaseJavaGenerator):
  """The GWT code generator."""

  def __init__(self, discovery, options=None):
    super(GwtGenerator, self).__init__(discovery, 'gwt',
                                       GwtLanguageModel(options=options),
                                       options=options)
    self.api.module.SetPath('%s/shared' % self.api.values['name'])

  def _InnerModelClassesSupported(self):
    """Gets whether or not inner model classes are supported."""
    return False

  def AnnotateApi(self, api):
    """Add GWT specific annotations to the Api dictionary."""
    super(GwtGenerator, self).AnnotateApi(api)

  def AnnotateMethod(self, api, method, unused_resource):
    """Add GWT-specific annotations and naming schemes."""
    method.SetTemplateValue('className',
                            '%sRequest' % method.values['className'])
    super(GwtGenerator, self).AnnotateMethod(api, method, None)

  def AnnotateParameter(self, method, parameter):
    """Add GWT-specific annotations to parameter declaration.

    Qualify our enum class names by method so they do not clash.

    Args:
      method: (Method) The method this parameter belongs to.
      parameter: (Parameter) The parameter to annotate.
    """
    super(GwtGenerator, self).AnnotateParameter(method, parameter)
    enum_type = parameter.values.get('enumType')
    if enum_type:
      # For generated enums, we need to qualify the parent class of the enum so
      # that two methods that take a similarly-named enum parameter don't get
      # confused.
      # TODO(user): This will fail when enums can be $refs. Get rid of all
      # special purpuse enum annotations and checks.
      name = '%s.%s' % (method.values['className'],
                        enum_type.values['className'])
      enum_type.SetTemplateValue('codeType', name)

  def AnnotateResource(self, api, resource):
    """Add GWT-specific annotations and naming schemes."""
    super(GwtGenerator, self).AnnotateResource(api, resource)
    parent_list = resource.ancestors[1:]
    parent_list.append(resource)
    parent_classes = [p.values.get('className') for p in parent_list]
    resource.SetTemplateValue('contextCodeType', '.'.join(parent_classes))
    # for the codeName, we do not lowercase my code name
    code_name = '_'.join(parent_classes[0:-1]).lower()
    if code_name:
      code_name += '_'
    code_name += resource.codeName
    resource.SetTemplateValue('contextCodeName', code_name)


class GwtLanguageModel(java_generator.JavaLanguageModel):
  """A LanguageModel for GWT."""

  language = 'java'

  member_policy = language_model.NamingPolicy(language_model.LOWER_CAMEL_CASE)
  getter_policy = language_model.NamingPolicy(
      language_model.UPPER_CAMEL_CASE, 'get{name}')
  setter_policy = language_model.NamingPolicy(
      language_model.UPPER_CAMEL_CASE, 'set{name}')
  constant_policy = language_model.NamingPolicy(
      case_transform=language_model.UPPER_UNCAMEL_CASE,
      separator='_')
  array_of_policy = language_model.NamingPolicy(
      format_string='java.util.List<{name}>')
  map_of_policy = language_model.NamingPolicy(
      format_string='java.util.Map<String, {name}>')

  # Dictionary of json type and format to its corresponding import definition.
  # The first import in the imports list is the primary import.
  TYPE_FORMAT_TO_DATATYPE_AND_IMPORTS = {
      ('boolean', None): ('Boolean', ImportDefinition(['java.lang.Boolean'])),
      # Use String instead of Object for GWT
      ('any', None): ('String', ImportDefinition(['java.lang.String'])),
      ('integer', 'int16'): ('Short', ImportDefinition(['java.lang.Short'])),
      ('integer', 'int32'): ('Integer',
                             ImportDefinition(['java.lang.Integer'])),
      # Java does not support Unsigned Integers
      ('integer', 'uint32'): ('Long', ImportDefinition(['java.lang.Long'])),
      ('number', 'double'): ('Double', ImportDefinition(['java.lang.Double'])),
      ('number', 'float'): ('Float', ImportDefinition(['java.lang.Float'])),
      # Use Splittable instead of Object for GWT
      ('object', None): ('Splittable', ImportDefinition(
          ['com.google.web.bindery.autobean.shared.Splittable'])),
      ('string', None): ('String', ImportDefinition(['java.lang.String'])),
      ('string', 'byte'): ('String', ImportDefinition(['java.lang.String'])),
      # GWT does not support date-time
      ('string', 'date-time'): ('String',
                                ImportDefinition(['java.lang.String'])),
      ('string', 'int64'): ('Long', ImportDefinition(['java.lang.Long'])),
      # Java does not support Unsigned Integers
      ('string', 'uint64'): ('BigInteger',
                             ImportDefinition(['java.math.BigInteger'])),
      }

  def __init__(self, options=None):
    super(GwtLanguageModel, self).__init__(options=options)
    self._type_map = GwtLanguageModel.TYPE_FORMAT_TO_DATATYPE_AND_IMPORTS

  def CodeTypeForVoid(self):
    """Return a GWT style void.

    Overrides the default.

    Returns:
      (str) 'EmptyResponse'
    """
    # TODO(user): Handle empty responses in a more graceful way. Instead of
    # passing the type to use for empty responses, just signal that an empty
    # response is required, and handle that in the templates.
    return 'EmptyResponse'

  def DefaultContainerPathForOwner(self, module):
    """Overrides the default implementation."""
    return '%s/api/gwt/services' % '/'.join(
        utilities.ReversedDomainComponents(module.owner_domain))



#!/usr/bin/python2.7
# Copyright 2011 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for php_generator."""

__author__ = 'chirags@google.com (Chirag Shah)'



from google.apputils import basetest
from googleapis.codegen import api
from googleapis.codegen import php_generator
from googleapis.codegen import schema


class PHPApiTest(basetest.TestCase):

  def setUp(self):
    gen_params = {'name': 'test', 'version': 'v1', 'resources': {}}
    self.api = php_generator.PHPApi(gen_params)
    self.generator = php_generator.PHPGenerator(gen_params)
    self.language_model = php_generator.PhpLanguageModel()
    # TODO(user): Do what we did for template_helpers and allow language
    # model to come from global state. Then we don't need this stuff.
    self.api.VisitAll(lambda o: o.SetLanguageModel(self.language_model))

  def tearDown(self):
    self.api = None

  def testAnnotateMethod(self):
    param_dict = {'httpMethod': 'GET',
                  'id': 'myservice.foo.count',
                  'parameters': {'alt': {}}}
    method = api.Method(self.api, 'count', param_dict)
    self.generator.AnnotateMethod(self.api, method)

    self.assertEquals('myservice.foo.count', method.values['id'])
    self.assertEquals('count', method.values['name'])
    self.assertEquals('count', method.values['wireName'])
    self.assertEquals('Count', method.values['className'])

  def testSetTypeHint(self):
    """Test creating safe class names from object names."""
    test_schema = api.Schema(self.api, 'testSchema', {})
    type_to_hint = [
        ({'$ref': 'Activity'}, 'Google_Service_Test_Activity'),
        ({'type': 'boolean'}, ''),
        ({'type': 'integer'}, ''),
        ({'type': 'string'}, ''),  # PHP doesn't support primitive type hints.
        ({'type': 'StRing'}, ''),  # PHP var names are case-insensitive.
        ({'$ref': 'Photo'}, 'Google_Service_Test_Photo'),
        ({'type': 'array', 'items': {'type': 'string'}}, ''),
        ({'type': 'object', 'properties': {'p1': {'type': 'string'}}},
         'Google_Service_Test_TestSchemaTest'),
        ]
    for type_dict, expected_hint in type_to_hint:
      test_property = schema.Property(self.api, test_schema, 'test', type_dict)
      test_property.SetLanguageModel(self.language_model)
      self.generator._SetTypeHint(test_property)
      self.assertEquals(expected_hint, test_property.values['typeHint'])

  def testToMethodName(self):
    """Test creating safe method names from wire names."""
    method = {'wireName': 'foo'}
    method_name = self.generator._ToMethodName(method, None)
    self.assertEquals('foo', method_name)

    # Method name that doesn't conflict with a PHP keyword.
    method['wireName'] = 'get'
    resource = {'className': 'ResourceClassName'}
    method_name = self.generator._ToMethodName(method, resource)
    self.assertEquals('get', method_name)

    # Method name that conflicts with a PHP keyword.
    method['wireName'] = 'as'
    resource['className'] = 'Class'
    method_name = self.generator._ToMethodName(method, resource)
    self.assertEquals('asClass', method_name)

    # Method name that conflicts with a canonical PHP keyword.
    method['wireName'] = 'aS'
    method_name = self.generator._ToMethodName(method, resource)
    self.assertEquals('aSClass', method_name)

  def testToClassName(self):
    """Test creating safe class names from object names."""
    self.assertEquals('Foo', self.api.ToClassName('foo', None))
    self.assertEquals('TestObject', self.api.ToClassName('object', None))
    self.assertEquals('TestString', self.api.ToClassName('string', None))

  def testGetCodeTypeFromDictionary(self):
    """Test mapping of JSON schema types to PHP class names."""

    php_type_to_schema = [('object', {'type': 'object'}),
                          ('string', {'type': 'string'}),
                          ('array', {'type': 'any'}),
                          ('bool', {'type': 'boolean'}),
                          ('int', {'type': 'integer'}),

                          ('string', {'type': 'number', 'format': 'uint32'}),
                          ('string', {'type': 'integer', 'format': 'uint32'}),
                          ('string', {'type': 'string', 'format': 'uint32'}),

                          ('string', {'type': 'number', 'format': 'uint64'}),
                          ('string', {'type': 'integer', 'format': 'uint64'}),
                          ('string', {'type': 'string', 'format': 'uint64'}),

                          ('int', {'type': 'number', 'format': 'int32'}),
                          ('int', {'type': 'integer', 'format': 'int32'}),
                          ('int', {'type': 'string', 'format': 'int32'}),

                          ('string', {'type': 'number', 'format': 'int64'}),
                          ('string', {'type': 'integer', 'format': 'int64'}),
                          ('string', {'type': 'string', 'format': 'int64'}),

                          ('string', {'type': 'string',
                                      'format': 'date-time'}),

                          ('double', {'type': 'number', 'format': 'double'}),
                          ('double', {'type': 'string', 'format': 'double'}),

                          ('float', {'type': 'number', 'format': 'float'}),
                          ('float', {'type': 'string', 'format': 'float'})]

    for schema_obj in php_type_to_schema:
      php_type = schema_obj[0]
      s = schema_obj[1]
      self.assertEquals(php_type,
                        self.language_model.GetCodeTypeFromDictionary(s))

if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
# Copyright 2013 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Sample library generator.

This module generates code from a Google API discovery documents.
"""

from googleapis.codegen import api
from googleapis.codegen import api_library_generator
from googleapis.codegen import language_model
from googleapis.codegen import utilities


class SampleLanguageModel(language_model.LanguageModel):
  """A LanguageModel for a sample language."""

  language = 'sample'

  # Set up variable naming policy
  allowed_characters = '_'
  array_of_policy = language_model.NamingPolicy(
      format_string='{module}::Array<{name}>')
  map_of_policy = language_model.NamingPolicy(
      format_string='Map<string, {name}>')
  class_name_policy = language_model.NamingPolicy(
      case_transform=language_model.LOWER_CASE, separator='_')
  constant_policy = language_model.NamingPolicy(
      case_transform=language_model.UPPER_CASE, separator='_')
  member_policy = language_model.NamingPolicy(
      format_string='_{name}', separator='_')
  getter_policy = language_model.NamingPolicy(
      case_transform=language_model.UPPER_CAMEL_CASE,
      format_string='get{name}')
  setter_policy = language_model.NamingPolicy(
      case_transform=language_model.UPPER_CAMEL_CASE,
      format_string='set{name}')
  parameter_name_policy = language_model.NamingPolicy(
      format_string='{name}', separator='_')

  _SCHEMA_TYPE_TO_LANGUAGE_TYPE = {
      'any': 'object',
      'array': 'array',
      'boolean': 'bool',
      'integer': 'long',
      'number': 'float',
      'object': 'object',
      'string': 'str',
      }

  # We can not create classes which match a keyword or built in object type.
  RESERVED_CLASS_NAMES = ['class', 'import', 'return']  # ...

  def __init__(self):
    super(SampleLanguageModel, self).__init__(module_name_delimiter='::')

    self._SUPPORTED_TYPES['array'] = self._List
    self._SUPPORTED_TYPES['boolean'] = self._Boolean
    self._SUPPORTED_TYPES['object'] = self._Dictionary

  def _Boolean(self, data_value):
    """Convert provided boolean to language specific literal."""
    return unicode(bool(data_value.value))

  def _Dictionary(self, data_value):
    """Convert provided object to language specific literal."""
    wrapper = '{%s}'
    pairs = []

    for key, val in data_value.value.iteritems():
      val = self.RenderDataValue(val)
      pairs.append('"%s": %s' % (key, val))

    return wrapper % ', '.join(pairs)

  def _List(self, data_value):
    """Convert provided array to language specific literal."""
    wrapper = '[%s]'
    items = [self.RenderDataValue(element) for element in data_value.value]

    return wrapper % ', '.join(items)

  def GetCodeTypeFromDictionary(self, def_dict):
    """Gets an element's data type from its JSON definition.

    Overrides the default.

    Args:
      def_dict: (dict) The definition dictionary for this type
    Returns:
      A name suitable for use as a data type
    """
    json_type = def_dict.get('type', 'string')
    native_type = (self._SCHEMA_TYPE_TO_LANGUAGE_TYPE.get(json_type) or
                   self._SCHEMA_TYPE_TO_LANGUAGE_TYPE.get('string'))
    return native_type

  def ToMemberName(self, s, unused_api):
    """Convert a wire format name into a suitable variable name."""
    return s.replace('-', '_')


class SampleGenerator(api_library_generator.ApiLibraryGenerator):
  """The Sample code generator."""

  def __init__(self, discovery_doc, options=None):
    super(SampleGenerator, self).__init__(
        SampleApi,
        discovery_doc,
        language='sample',
        language_model=SampleLanguageModel(),
        options=options)


class SampleApi(api.Api):
  """An Api with annotations for this language."""

  def __init__(self, discovery_doc, **unused_kwargs):
    super(SampleApi, self).__init__(discovery_doc)

  # pylint: disable=unused-argument
  def ToClassName(self, s, element, element_type=None):
    """Convert a discovery name to a suitable class name.

    Overrides the default.

    Args:
      s: (str) A rosy name of data element.
      element: (object) The object we need a class name for.
      element_type: (str) The kind of element (resource|method) to name.
    Returns:
      A name suitable for use as a class in the generator's target language.
    """

    if s.lower() in SampleLanguageModel.RESERVED_CLASS_NAMES:
      # Prepend the service name
      return '%s%s' % (utilities.CamelCase(self.values['name']),
                       utilities.CamelCase(s))
    return utilities.CamelCase(s)



#!/usr/bin/python2.7
# Copyright 2010 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""The Exceptions we can raise from API parsing."""


class ApiException(Exception):
  """The base class for all API parsing exceptions."""

  def __init__(self, reason, def_dict=None):
    """Create an exception.

    Args:
      reason: (str) The human readable explanation of this exception.
      def_dict: (dict) The discovery dictionary we failed on.
    """
    super(ApiException, self).__init__()
    self._reason = reason
    self._def_dict = def_dict

  def __str__(self):
    if self._def_dict:
      return '%s: %s' % (self._reason, self._def_dict)
    return self._reason



#!/usr/bin/python2.7
"""Setup script for Google APIs Client Generator."""

import setuptools


setuptools.setup(
    name='google-apis-client-generator',
    version='1.6.1',
    description='Google Apis Client Generator',
    long_description=open('README').read(),
    author='Google Inc.',
    author_email='opensource-apis-client-generator@google.com',
    url='https://github.com/google/apis-client-generator/',
    packages=setuptools.find_packages('src'),
    package_dir={'': 'src'},
    entry_points={
        'console_scripts': [
            ('generate_library = '
             'googleapis.codegen.script_stubs:RunGenerateLibrary'),
            ('expand_templates = '
             'googleapis.codegen.script_stubs:RunExpandTemplates')
            ]},
    include_package_data=True,
    install_requires=['django>=1.4.0, <1.9.0dev',
                      'httplib2>=0.9.2, <2.0.0dev',
                      'google-apputils>=0.4.2, <2.0.0dev',
                      'python-gflags>=2.0, <4.0.0dev',
                      'google-api-python-client>=1.0.0, <2.0.0dev'],
    zip_safe=False)



#!/usr/bin/python2.7
# Copyright 2010 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for schema.py."""

__author__ = 'aiuto@google.com (Tony Aiuto)'


from google.apputils import basetest

from googleapis.codegen import language_model
from googleapis.codegen.api import Api
from googleapis.codegen.api_exception import ApiException
from googleapis.codegen.schema import Schema


class FakeLanguageModel(language_model.LanguageModel):

  language = 'fake'

  def GetCodeTypeFromDictionary(self, def_dict):
    return def_dict.get('type')

  def ArrayOf(self, unused_var, s):
    return 'Array[%s]' % s

  def ToMemberName(self, s, unused_api):
    return s.replace('-', '_')


def MakeApiWithSchemas(schemas):
  discovery_doc = {
      'name': 'fake',
      'version': 'v1',
      'resources': {},
      'schemas': schemas
      }
  api = Api(discovery_doc)
  api.VisitAll(lambda o: o.SetLanguageModel(FakeLanguageModel()))
  return api


class SchemaTest(basetest.TestCase):
  """Tests for the Schema class."""

  def testArrayOfArray(self):
    api = MakeApiWithSchemas({
        'AdsenseReportsGenerateResponse': {
            'id': 'AdsenseReportsGenerateResponse',
            'type': 'object',
            'properties': {
                'basic': {
                    'type': 'string'
                    },
                'simple_array': {
                    'type': 'array',
                    'items': {'type': 'string'}
                    },
                'array_of_arrays': {
                    'type': 'array',
                    'items': {'type': 'array', 'items': {'type': 'string'}}
                    }
                }
            }
        })
    response_schema = api._schemas.get('AdsenseReportsGenerateResponse')
    self.assertTrue(response_schema)
    prop = [prop for prop in response_schema.values['properties']
            if prop.values['wireName'] == 'array_of_arrays']
    self.assertTrue(len(prop) == 1)
    prop = prop[0]
    self.assertEquals('Array[Array[string]]', prop.codeType)

  def testDetectInvalidSchema(self):
    bad_discovery = {
        'name': 'fake',
        'version': 'v1',
        'resources': {},
        'schemas': {
            'NoItemsInArray': {'id': 'noitems', 'type': 'array'}
            }
        }
    self.assertRaises(ApiException, Api, bad_discovery)

  def testSchemaWithoutProperties(self):
    api = MakeApiWithSchemas({
        'NoProperties': {'id': 'NoProperties', 'type': 'object'}
        })
    for name, schema in api._schemas.items():
      if name == 'NoProperties':
        self.assertEquals(0, len(schema.values.get('properties')))
        return
    self.fail('Did not find NoProperties')

  def testSchemaWithAdditionalPropertiesWithoutId(self):
    api = MakeApiWithSchemas({
        'Snorg': {
            'id': 'Snorg',
            'type': 'object',
            'additionalProperties': {
                'type': 'object',
                'properties': {
                    'thing': {
                        'type': 'boolean'
                        }
                    }
                }
            },
        'SnorgFresser': {
            'id': 'SnorgFresser',
            'type': 'object',
            'properties': {
                'snacks': {
                    'type': 'array',
                    'items': {
                        '$ref': 'Snorg'
                        }
                    }
                }
            }
        })
    schemas = api._schemas
    self.assertTrue('SnorgFresser' in schemas)
    self.assertTrue('Snorg' in schemas)
    self.assertTrue('SnorgElement' in schemas)
    snorg = api.SchemaByName('Snorg')
    self.assertTrue(snorg)
    self.assertFalse('Snorg' in api.ModelClasses())
    snorg_element = api.SchemaByName('SnorgElement')
    self.assertTrue(snorg_element)
    self.assertTrue(snorg_element in api.ModelClasses())

  def testNestedSchemaWithAdditionalProperties(self):
    api = MakeApiWithSchemas({
        'RestDescription': {
            'id': 'RestDescription',
            'type': 'object',
            'properties': {
                'auth': {
                    'type': 'object',
                    'properties': {
                        'oauth2': {
                            'type': 'object',
                            'properties': {
                                'scopes': {
                                    'type': 'object',
                                    'additionalProperties': {
                                        'type': 'object',
                                        'properties': {
                                            'description': {
                                                'type': 'string',
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    },
                }
            }
        })

    expected_names = {'RestDescription',
                      'RestDescriptionAuth',
                      'RestDescriptionAuthOauth2',
                      'RestDescriptionAuthOauth2Scopes',
                      'RestDescriptionAuthOauth2ScopesElement'}
    schema_names = set(x.values.get('className') for x
                       in api._schemas.itervalues())
    self.assertEquals(expected_names, schema_names)
    scopes_elem = api._schemas['RestDescription.auth.oauth2.scopesElement']
    self.assertEquals('ScopesElement', scopes_elem.safe_code_type)
    self.assertEquals('RestDescriptionAuthOauth2ScopesElement',
                      scopes_elem.code_type)
    oauth2_elem = api._schemas['RestDescription.auth.oauth2']
    self.assertEquals('Oauth2', oauth2_elem.safe_code_type)
    self.assertEquals('RestDescriptionAuthOauth2', oauth2_elem.code_type)

  def testSchemaWithAdditionalPropertiesWithId(self):
    api = MakeApiWithSchemas({
        'Snorg': {
            'id': 'Snorg',
            'type': 'object',
            'additionalProperties': {
                'id': 'Skrimpkin',
                'type': 'object',
                'properties': {
                    'thing': {
                        'type': 'boolean'
                        }
                    }
                }
            },
        'SnorgFresser': {
            'id': 'SnorgFresser',
            'type': 'object',
            'properties': {
                'snacks': {
                    'type': 'array',
                    'items': {
                        '$ref': 'Snorg'
                        }
                    }
                }
            }
        })
    schemas = api._schemas
    self.assertTrue('SnorgFresser' in schemas)
    self.assertTrue('Snorg' in schemas)
    self.assertTrue('Skrimpkin' in schemas)
    snorg = api.SchemaByName('Snorg')
    self.assertTrue(snorg)
    self.assertFalse('Snorg' in api.ModelClasses())
    skrimpkin = api.SchemaByName('Skrimpkin')
    self.assertTrue(skrimpkin)
    self.assertTrue(skrimpkin in api.ModelClasses())

  def testUndefinedSchema(self):
    gen = MakeApiWithSchemas({
        'foo': {
            'id': 'foo',
            'type': 'object',
            'properties': {'basic': {'$ref': 'bar'}}
            }
        })
    # We expect foo to be in the list because the id is 'foo'
    self.assertTrue('foo' in gen._schemas.keys())
    # We expect 'Foo' to be in the list because that is the class name we would
    # create for foo
    self.assertTrue('foo' in gen._schemas.keys())
    # We do not expect Bar to be in the list because we only have a ref to it
    # but no definition.
    self.assertFalse('Bar' in gen._schemas.keys())

  def testSchemaWithNameClash(self):
    clashing_names = {
        'Snorg': {
            'id': 'Snorg',
            'type': 'object',
            'properties': {
                'thing': {
                    'type': 'boolean'
                },
                '@thing': {
                    'type': 'boolean'
                }
            }
        }
    }
    self.assertRaises(ApiException, MakeApiWithSchemas, clashing_names)

  def testWrappedContainer(self):
    discovery_doc = {
        'name': 'fake',
        'version': 'v1',
        }
    api = Api(discovery_doc)
    wrapped_container_def = {
        'id': 'SeriesList',
        'type': 'object',
        'properties': {
            'items': {
                'type': 'array',
                'items': {
                    '$ref': 'Snorg'
                    }
                },
            }
        }
    schema = Schema.Create(api, 'foo', wrapped_container_def, 'foo', None)
    self.assertEquals(1, len(schema.properties))
    self.assertIsNotNone(schema.isContainerWrapper)
    container_property = schema.containerProperty
    self.assertIsNotNone(container_property)
    array_of = container_property.data_type.GetTemplateValue('arrayOf')
    self.assertIsNotNone(array_of)
    self.assertEquals('Snorg', array_of.values['wireName'])

    # Add a kind
    wrapped_container_def['properties'].update({'kind': {'type': 'string'}})
    schema = Schema.Create(api, 'foo', wrapped_container_def, 'foo', None)
    self.assertEquals(2, len(schema.properties))
    self.assertTrue(schema.isContainerWrapper)

    # Add an etag
    wrapped_container_def['properties'].update({'etag': {'type': 'string'}})
    schema = Schema.Create(api, 'foo', wrapped_container_def, 'foo', None)
    self.assertEquals(3, len(schema.properties))
    self.assertTrue(schema.isContainerWrapper)

    # Add a field which disqualifies
    wrapped_container_def['properties'].update({'foo': {'type': 'string'}})
    schema = Schema.Create(api, 'foo', wrapped_container_def, 'foo', None)
    self.assertEquals(4, len(schema.properties))
    self.assertFalse(schema.isContainerWrapper)

    # Make the main property not a container
    not_wrapped_container_def = {
        'id': 'SeriesList',
        'type': 'object',
        'properties': {
            'items': {'type': 'string'},
            'kind': {'type': 'string'}
            }
        }
    schema = Schema.Create(api, 'foo', not_wrapped_container_def, 'foo', None)
    self.assertEquals(2, len(schema.properties))
    self.assertFalse(schema.isContainerWrapper)

  def testMemberNameIsJsonName(self):
    api = MakeApiWithSchemas({
        's': {
            'type': 'object',
            'properties': {
                'easyname': {
                    'type': 'string',
                    'expect': True,
                    },
                'dashed-name': {
                    'type': 'string',
                    'expect': False,
                    }
                }
            }
        })
    for name, schema in api._schemas.items():
      if name == 's':
        expect_len = len(schema.values.get('properties'))
        got = 0
        for p in schema.values.get('properties'):
          self.assertEquals(
              p.values['expect'], p.member_name_is_json_name,
              'Unexpected code name %s for json name: %s' % (
                  p.memberName, p.values['wireName']))
          got += 1
        self.assertEquals(expect_len, got)
        return
    self.fail('Did not find schema')


if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
#
# Copyright 2010 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Java library generator's import manager."""

__author__ = 'rmistry@google.com (Ravi Mistry)'


# Used to cache import managers to the class name of the schema they
# correspond to.
_CLASS_NAME_TO_IMPORT_MANAGER = {}


class JavaImportManager(object):
  """The import manager for the Java code generator."""

  _JAVA_LANG_IMPORT = 'java.lang'

  def __init__(self, element):
    """Construct an import manager for the specified element.

    Args:
      element: (Schema) or (Api). The element we want to create an import
        manager for.
    """
    self._element = element
    self._element.SetTemplateValue('importManager', self)
    self._class_name_to_qualified_name = {}
    self._google_imports = set()
    self._other_imports = set()
    self._java_imports = set()

  @classmethod
  def GetCachedImportManager(cls, element):
    """Gets an import manager instance that corresponds to the class name.

    If the schema does not have a cached import manager, one is created
    and added to the cache.

    Args:
      element: (Schema) or (Api). The element we want to create an import
        manager for.
    Returns:
      The import manager instance for this schema.
    """
    import_mngr = _CLASS_NAME_TO_IMPORT_MANAGER.get(element)
    if not import_mngr:
      # This class does not have an import manager yet. Instantiate it.
      import_mngr = cls(element)
      _CLASS_NAME_TO_IMPORT_MANAGER[element] = import_mngr
    return import_mngr

  def ImportLists(self):
    """Returns the set of import lists."""
    return [sorted(self._google_imports),
            sorted(self._java_imports),
            sorted(self._other_imports)]

  def AddImport(self, fully_qualified_class):
    """Adds the specified import to the import manager.

    Args:
      fully_qualified_class: (str) The fully qualified class we want to add to
        this instance of import manager.

    Returns:
      True- If adding the import is successful.
      False- If adding the import results in a name collision.
    """
    if fully_qualified_class.startswith(self._JAVA_LANG_IMPORT):
      # Returning True because it is imported by default.
      return True

    # check to see if it already exists
    class_name = self.GetClassName(fully_qualified_class)
    current_import = self._class_name_to_qualified_name.get(
        class_name)
    if current_import:
      return current_import == fully_qualified_class

    if fully_qualified_class.startswith('com.google.'):
      self._google_imports.add(fully_qualified_class)
    elif fully_qualified_class.startswith('java.'):
      self._java_imports.add(fully_qualified_class)
    else:
      self._other_imports.add(fully_qualified_class)
    # Now add it to the map
    self._class_name_to_qualified_name[class_name] = fully_qualified_class
    return True

  def GetClassName(self, fully_qualified_class):
    return fully_qualified_class.split('.')[-1]

  @property
  def google_imports(self):
    return sorted(self._google_imports)

  @property
  def other_imports(self):
    return sorted(self._other_imports)

  @property
  def java_imports(self):
    return sorted(self._java_imports)



#!/usr/bin/python2.7
# Copyright 2010 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Java library generator.

Specializations to the code Generator for Java bindings. This class also serves
as a superclass for GWT generation, since that shares Java's naming rules.
"""

__author__ = 'aiuto@google.com (Tony Aiuto)'

from googleapis.codegen import api
from googleapis.codegen import api_library_generator
from googleapis.codegen import data_types
from googleapis.codegen import language_model
from googleapis.codegen import utilities
from googleapis.codegen.import_definition import ImportDefinition
from googleapis.codegen.java_import_manager import JavaImportManager
from googleapis.codegen.utilities import maven_utils


class BaseJavaGenerator(api_library_generator.ApiLibraryGenerator):
  """Base for Java code generators."""

  # pylint: disable=redefined-outer-name
  def __init__(self, discovery, language='java', language_model=None,
               options=None):
    if not language_model:
      language_model = self._GetDefaultLanguageModel(options)
    super(BaseJavaGenerator, self).__init__(JavaApi, discovery, language,
                                            language_model, options=options)

  @classmethod
  def _GetDefaultLanguageModel(cls, options=None):
    return JavaLanguageModel(options=options)

  def _InnerModelClassesSupported(self):
    """Gets whether or not inner model classes are supported."""
    return True

  def AnnotateApi(self, the_api):
    """Annotate the Api dictionary with Java specifics."""
    the_api.SetTemplateValue(
        'maven',
        maven_utils.GetMavenMetadata(the_api, self.language_version),
        'maven metadata')

  def AnnotateParameter(self, unused_method, parameter):
    """Annotate a Parameter with Java specific elements."""
    self._HandleImports(parameter)

  def AnnotateProperty(self, unused_api, prop, unused_schema):
    """Annotate a Property with Java specific elements."""
    self._HandleImports(prop)

  def _HandleImports(self, element):
    """Handles imports for the specified element.

    Args:
      element: (Property|Parameter) The property we want to set the import for.
    """
    # Get the parent of this Property/Parameter.
    parent = element.schema
    if self._InnerModelClassesSupported():
      # If inner classes are supported find the top level parent to set
      # imports for.
      while parent.parent:
        parent = parent.parent
    import_manager = JavaImportManager.GetCachedImportManager(parent)

    # For collections, we have to get the imports for the first non-collection
    # up the base_type chain.
    data_type = element.data_type
    json_type = data_type.json_type
    while json_type == 'array' or json_type == 'map':
      data_type = data_type._base_type  # pylint: disable=protected-access
      json_type = data_type.json_type
    json_format = data_type.values.get('format')

    type_format_to_datatype_and_imports = self.language_model.type_map
    datatype_and_imports = (
        type_format_to_datatype_and_imports.get((json_type, json_format)))
    if datatype_and_imports:
      import_definition = datatype_and_imports[1]
      # Import all required imports.
      for required_import in import_definition.imports:
        import_manager.AddImport(required_import)
      # Set all template values, if specified.
      for template_value in import_definition.template_values:
        element.SetTemplateValue(template_value, True)


class Java14Generator(BaseJavaGenerator):
  """A Java generator for language version 1.14 and higher."""

  @classmethod
  def _GetDefaultLanguageModel(cls, options=None):
    return JavaLanguageModel(options=options)


def _SimpleType(class_name, primative_type=None):
  """Type definition for classes with no primitives or special imports."""
  return (class_name, ImportDefinition([class_name]), primative_type)


class JavaLanguageModel(language_model.LanguageModel):
  """A LanguageModel tuned for Java."""

  language = 'java'

  _JSON_STRING_IMPORT = 'com.google.api.client.json.JsonString'
  _JSON_STRING_TEMPLATE_VALUE = 'requiresJsonString'
  member_policy = language_model.NamingPolicy(language_model.LOWER_CAMEL_CASE)
  getter_policy = language_model.NamingPolicy(
      language_model.UPPER_CAMEL_CASE, 'get{name}')
  setter_policy = language_model.NamingPolicy(
      language_model.UPPER_CAMEL_CASE, 'set{name}')
  has_policy = language_model.NamingPolicy(
      language_model.UPPER_CAMEL_CASE, 'has{name}')
  array_of_policy = language_model.NamingPolicy(
      format_string='java.util.List<{name}>')
  map_of_policy = language_model.NamingPolicy(
      format_string='java.util.Map<String, {name}>')
  constant_policy = language_model.NamingPolicy(
      case_transform=language_model.UPPER_UNCAMEL_CASE,
      separator='_')
  enum_policy = language_model.NamingPolicy(
      language_model.UPPER_CAMEL_CASE,
      format_string='{name}Values')

  # Dictionary of json type and format to its corresponding data type and
  # import definition. The first import in the imports list is the primary
  # import.
  _TYPE_BOOLEAN = _SimpleType('java.lang.Boolean', primative_type='boolean')
  _TYPE_DOUBLE = _SimpleType('java.lang.Double', primative_type='double')
  _TYPE_FLOAT = _SimpleType('java.lang.Float', primative_type='float')
  _TYPE_INTEGER = _SimpleType('java.lang.Integer', primative_type='int')
  _TYPE_LONG = _SimpleType('java.lang.Long', primative_type='long')
  _TYPE_SHORT = _SimpleType('java.lang.Short', primative_type='short')
  _TYPE_DATETIME = ('com.google.api.client.util.DateTime',
                    ImportDefinition(['com.google.api.client.util.DateTime']),
                    None)
  _TYPE_STRING = _SimpleType('java.lang.String')

  _TYPE_FORMAT_TO_DATATYPE_AND_IMPORTS = {
      ('any', None): _SimpleType('java.lang.Object'),
      ('boolean', None): _TYPE_BOOLEAN,
      ('integer', None): _TYPE_INTEGER,
      ('integer', 'int16'): _TYPE_SHORT,
      ('integer', 'int32'): _TYPE_INTEGER,
      # We prefer Long here over UnsignedInteger because Long has built-in
      # support for autoboxing in Java.
      ('integer', 'uint32'): _TYPE_LONG,
      # TODO(user): Switch to java.lang.Number in a separate CL.
      # ('number', None): _SimpleType('java.lang.Number'),
      ('number', None): ('Number', ImportDefinition(), None),
      ('number', 'double'): _TYPE_DOUBLE,
      ('number', 'float'): _TYPE_FLOAT,
      ('object', None): _SimpleType('java.lang.Object'),
      ('string', None): _TYPE_STRING,
      ('string', 'byte'): _TYPE_STRING,
      ('string', 'date'): _TYPE_DATETIME,
      ('string', 'date-time'): _TYPE_DATETIME,
      ('string', 'int64'): ('java.lang.Long',
                            ImportDefinition(
                                ['java.lang.Long', _JSON_STRING_IMPORT],
                                [_JSON_STRING_TEMPLATE_VALUE]),
                            'long'),
      ('string', 'uint64'): ('java.math.BigInteger',
                             ImportDefinition(
                                 ['java.math.BigInteger', _JSON_STRING_IMPORT],
                                 [_JSON_STRING_TEMPLATE_VALUE]),
                             None),
      }

  _JAVA_KEYWORDS = [
      'abstract', 'assert', 'boolean', 'break', 'byte', 'case', 'catch', 'char',
      'class', 'const', 'continue', 'default', 'do', 'double', 'else', 'enum',
      'extends', 'final', 'finally', 'float', 'for', 'goto', 'if', 'implements',
      'import', 'instanceof', 'int', 'interface', 'long', 'native', 'new',
      'package', 'private', 'protected', 'public', 'return', 'short', 'static',
      'strictfp', 'super', 'switch', 'synchronized', 'this', 'throw', 'throws',
      'transient', 'try', 'void', 'volatile', 'while'
      ]

  # TODO(user): Remove this. We should instead be using fully qualified names
  # for models to avoid collisions. This will be fixed in Bug 6448720.
  _SPECIAL_CLASS_NAMES = [
      # Required because GenericData extends AbstractMap<String, Object>
      'entry'
      ]

  # We can not create classes which match a Java keyword or built in object
  # type.
  RESERVED_CLASS_NAMES = _JAVA_KEYWORDS + _SPECIAL_CLASS_NAMES + [
      'float', 'integer', 'object', 'string', 'true', 'false', 'null'
      ]

  allowed_characters = '$'
  reserved_words = RESERVED_CLASS_NAMES

  def __init__(self, options=None):
    super(JavaLanguageModel, self).__init__(class_name_delimiter='.')
    self._type_map = JavaLanguageModel._TYPE_FORMAT_TO_DATATYPE_AND_IMPORTS

    self._SUPPORTED_TYPES['boolean'] = self._Boolean
    self._SUPPORTED_TYPES['integer'] = self._Int
    self._options = options

  def _Boolean(self, data_value):
    """Convert provided boolean to language specific literal."""
    return unicode(bool(data_value.value)).lower()

  def _Int(self, data_value):
    """Convert provided int to language specific literal."""
    # Available types can be found in class variables
    code_types = {
        'java.lang.Short': '%s',
        'java.lang.Integer': '%s',
        'java.lang.Long': '%sL',
    }
    try:
      return code_types[data_value.code_type] % long(data_value.value)
    except KeyError:
      raise ValueError(
          ('Provided DataValue (%s) does not present an appropriate Java '
           'annotated code type (%s).') %
          (data_value.value, data_value.code_type))

  @property
  def type_map(self):
    return self._type_map

  def _GetTypeInfo(self, def_dict):
    """Returns a tuple of type information for a json schema data dict.

    For the dictionary containing {type, [format]}, return the tuple of the form
      (Java class name, ImportDefinition, Java primitive data type)
    describing the appropriate Java data type.

    Args:
      def_dict: (dict) JSON schema data definition.
    Returns:
      tuple of (class name, ImportDefinition, primitive data type).
    """
    json_type = def_dict.get('type', 'string')
    json_format = def_dict.get('format')
    result = self.type_map.get((json_type, json_format))
    if result:
      return result

    # TODO(user): Uncomment this and update golden files.
    # result = self.type_map.get((json_type, None))
    # if result:
    #   return result
    #
    # raise ValueError('Unknown type: %s format: %s' % (json_type, json_format))

    return (utilities.CamelCase(json_type), None, None)

  def GetCodeTypeFromDictionary(self, def_dict):
    """Convert a json schema type to a suitable Java type name.

    Overrides the default.

    Args:
      def_dict: (dict) A dictionary describing Json schema for this Property.
    Returns:
      A name suitable for use as a class in the generator's target language.
    """
    return self._GetTypeInfo(def_dict)[0]

  def GetPrimitiveTypeFromDictionary(self, def_dict):
    """Convert a json schema type to a suitable Java primitive name or None.

    Overrides the default.

    Args:
      def_dict: (dict) A dictionary describing Json schema for this Property.
    Returns:
      A name suitable for use as a primitive in the generator's target language.
    """
    return self._GetTypeInfo(def_dict)[2]

  def ToMemberName(self, s, the_api):
    """CamelCase a wire format name into a suitable Java variable name."""
    camel_s = utilities.CamelCase(s)
    if s.lower() in JavaLanguageModel.RESERVED_CLASS_NAMES:
      # Prepend the service name
      return '%s%s' % (the_api.values['name'], camel_s)
    return camel_s[0].lower() + camel_s[1:]

  def ToSafeClassName(self, s, the_api, parent=None):
    """Convert a name to a suitable class name in Java.

    Subclasses should override as appropriate.

    Args:
      s: (str) A canonical name for data element. (Usually the API wire format)
      the_api: (Api) The API this element is part of. For use as a hint when the
        name cannot be used directly.
      parent: (schema) The parent we use to get a safe class name.
    Returns:
      A name suitable for use as an element in Java.
    """
    safe_class_name = utilities.CamelCase(s)
    if parent:
      for ancestor in parent.full_path:
        if ancestor.safeClassName == safe_class_name:
          safe_class_name = '%s%s' % (parent.class_name, safe_class_name)
    if safe_class_name.lower() in JavaLanguageModel.RESERVED_CLASS_NAMES:
      api_name = utilities.CamelCase(the_api.values['name'])
      if utilities.CamelCase(the_api.values['canonicalName']):
        # Use canonical name if specified.
        api_name = utilities.CamelCase(
            the_api.values['canonicalName']).replace(' ', '')
      # Prepend the API name
      safe_class_name = '%s%s' % (api_name, utilities.CamelCase(s))
    return safe_class_name

  def ToPropertyGetterMethodWithDelim(self, prop_name):
    """Convert a property name to the name of the getter method that returns it.

    Args:
      prop_name: (str) The name of a property.
    Returns:
      A Java specific name of the getter method that returns the specified
      property. Eg: returns .getXyz for a property called xyz.
    """
    return '%sget%s()' % (self._class_name_delimiter,
                          utilities.CamelCase(prop_name))

  def CodeTypeForVoid(self):
    """Return the Java type name for a void.

    Overrides the default.

    Returns:
      (str) 'EmptyResponse'
    """
    return 'Void'

  def DefaultContainerPathForOwner(self, module):
    """Overrides the default."""
    return '/'.join(utilities.ReversedDomainComponents(module.owner_domain))


class JavaApi(api.Api):
  """An Api with Java annotations."""

  def __init__(self, discovery_doc, **unused_kwargs):
    super(JavaApi, self).__init__(discovery_doc)
    # Java puts the model classes in a module nested under the API, so adjust
    # the path accordingly.
    self.model_module.SetPath('model')

  def TopLevelModelClasses(self):
    """Return the models which are not children of another model.

    Adds an additional constraint on Api.TopLevelModelClasses to remove top
    level schemas which are arrays.

    Returns:
      list of Model.
    """
    return [m for m in self.ModelClasses()
            if not m.parent and not isinstance(m, data_types.ArrayDataType)]

  # pylint: disable=unused-argument
  def ToClassName(self, s, element, element_type=None):
    """Convert a discovery name to a suitable Java class name.

    In Java, nested class names cannot share the same name as the outer class
    name.

    Overrides the default.

    Args:
      s: (str) A rosy name of data element.
      element: (object) The object we need a class name for.
      element_type: (str) The kind of object we need a class name for.
    Returns:
      A name suitable for use as a class in the generator's target language.
    """
    # Camelcase what we have and account for spaces in canonical names
    name = utilities.CamelCase(s).replace(' ', '')
    if name.lower() in JavaLanguageModel.RESERVED_CLASS_NAMES:
      # Prepend the service name
      service = self._class_name or utilities.CamelCase(self.values['name'])
      return service + name

    if name == self.values.get('className'):
      if 'resource' == element_type:
        name += 'Operations'
      elif 'method' == element_type:
        name += 'Operation'
    return name



#!/usr/bin/python2.7


__author__ = 'jcgregorio@google.com (Joe Gregorio)'

import os

from google.apputils import basetest
from googleapis.codegen import targets


class BaseTargetsTest(basetest.TestCase):

  def setUp(self):
    testdata_dir = os.path.join(os.path.dirname(__file__), 'testdata')

    self.targets = targets.Targets(
        os.path.join(testdata_dir, 'targets_test.json'),
        os.path.join(testdata_dir, 'languages'))


class TargetsTest(BaseTargetsTest):

  def testTargetsAccessors(self):
    rawdata = self.targets.Dict()
    self.assertTrue('languages' in rawdata)

    self.assertTrue(
        self.targets.VariationsForLanguage('java').IsValid('preview'))
    self.assertTrue(
        self.targets.VariationsForLanguage('python').IsValid('stable'))
    self.assertTrue('displayName' in self.targets.GetLanguage('java'))
    self.assertTrue('python' in self.targets.Languages())
    self.assertTrue('cmd-line' in self.targets.Platforms())


class FeaturesLoadingTest(BaseTargetsTest):

  def testGetFeatures(self):
    variations = self.targets.VariationsForLanguage('java')
    features = variations.GetFeatures('preview')
    # Something from the top level
    self.assertEquals(True, features.get('library'))
    # Something overridden in the variation specific file
    self.assertEquals('base-client-library', features.get('baseClientLibrary'))
    self.assertEquals('release-version', features.get('releaseVersion'))

  def testGetFeaturesWithoutOverride(self):
    """Ask for features of a service without a local override."""
    variations = self.targets.VariationsForLanguage('java')
    features = variations.GetFeatures('not_built_in')
    self.assertEquals('this-is-from-top-level', features.get('releaseVersion'))
    self.assertIsNone(features.get('baseClientLibrary'))


class FeaturesTest(BaseTargetsTest):

  # short names for the various names in the test data
  base_path = 'libs/base.jar'
  base_source_path = 'libs-sources/base-sources.jar'
  base_docs_path = 'libs-sources/base-docs.jar'
  gae_path = 'libs/client-appengine.jar'
  gae_sources_path = 'libs-sources/client-appengine-sources.jar'
  logging_path = 'libs/logging.jar'
  servlet_path = 'libs/servlet.jar'
  servlet_source_path = 'libs-sources/servlet-sources.jar'

  def testDependenciesForEnvironment(self):
    variations = self.targets.VariationsForLanguage('java')
    features = variations.GetFeatures('preview')
    # short names for the various names in the test data
    nr = 'never required'
    base = 'base is always required'
    gae = 'required on gae'
    gae_generic = 'required on gae, but useful to all'
    gae_or_servlet = 'required on servlet or gae'

    def CheckResults(environment, expected_req, expected_opt, expected_exclude):
      req, opt, exclude = features.DependenciesForEnvironment(environment)
      self.assertSameElements(expected_req, [r['name'] for r in req])
      self.assertSameElements(expected_opt, [r['name'] for r in opt])
      self.assertSameElements(expected_exclude, [r['name'] for r in exclude])

    CheckResults(None,
                 [base], [gae, gae_generic, gae_or_servlet, nr], [])
    CheckResults('command_line',
                 [base], [gae_generic], [gae, gae_or_servlet, nr])
    CheckResults('appengine',
                 [base, gae, gae_generic, gae_or_servlet], [], [nr])
    CheckResults('servlet',
                 [base, gae_or_servlet], [gae_generic], [gae, nr])
    CheckResults('manual',
                 [base, nr], [gae_generic], [gae, gae_or_servlet])

  def testAllDependencyPaths(self):
    variations = self.targets.VariationsForLanguage('java')
    features = variations.GetFeatures('preview')
    all_paths = features.AllDependencyPaths()
    self.assertSameElements(
        [self.base_path, self.base_source_path, self.base_docs_path,
         self.logging_path, self.gae_path, self.gae_sources_path,
         self.servlet_path, self.servlet_source_path],
        all_paths)

  def testExtractPathsFromDependencies(self):
    variations = self.targets.VariationsForLanguage('java')
    features = variations.GetFeatures('preview')

    def CheckResults(environment, expected_req, expected_opt, expected_exclude):
      req, opt, exclude = features.DependenciesForEnvironment(environment)
      req_paths = features.ExtractPathsFromDependencies(req)
      opt_paths = features.ExtractPathsFromDependencies(opt)
      exclude_paths = features.ExtractPathsFromDependencies(exclude)
      self.assertSameElements(expected_req, req_paths)
      self.assertSameElements(expected_opt, opt_paths)
      self.assertSameElements(expected_exclude, exclude_paths)
      self.assertEquals(8, len(req_paths) + len(opt_paths) + len(exclude_paths))

    # Each test will account for all 8 file paths
    CheckResults(
        'command_line',
        [self.base_path, self.base_source_path, self.base_docs_path],
        [self.logging_path],
        [self.gae_path, self.gae_sources_path, self.servlet_path,
         self.servlet_source_path])
    CheckResults(
        'servlet',
        [self.base_path, self.base_source_path, self.base_docs_path,
         self.servlet_path, self.servlet_source_path],
        [self.logging_path],
        [self.gae_path, self.gae_sources_path])

    # Check the filtering
    req, _, _ = features.DependenciesForEnvironment('foo')
    bin_files = features.ExtractPathsFromDependencies(req, file_type='binary')
    self.assertSameElements([self.base_path], bin_files)

    req, _, _ = features.DependenciesForEnvironment('servlet')
    bin_files = features.ExtractPathsFromDependencies(req, file_type='binary')
    self.assertSameElements([self.base_path, self.servlet_path], bin_files)

  def testFilePathsWeDoNotDependOn(self):
    variations = self.targets.VariationsForLanguage('java')
    features = variations.GetFeatures('preview')

    all_paths = set([self.base_path, self.base_source_path, self.base_docs_path,
                     self.logging_path, self.gae_path, self.gae_sources_path,
                     self.servlet_path, self.servlet_source_path])

    exclude = features.FilePathsWeDoNotDependOn()
    self.assertSameElements([], exclude)

    exclude = features.FilePathsWeDoNotDependOn('foo')
    self.assertSameElements(
        all_paths - set([self.base_path, self.base_source_path,
                         self.base_docs_path]),
        exclude)

    exclude = features.FilePathsWeDoNotDependOn(file_type='binary')
    self.assertSameElements(all_paths - set([self.base_path]), exclude)

    exclude = features.FilePathsWeDoNotDependOn('foo', file_type='binary')
    self.assertSameElements(all_paths - set([self.base_path]), exclude)

    exclude = features.FilePathsWeDoNotDependOn('appengine', file_type='binary')
    self.assertSameElements(
        all_paths - set([self.base_path, self.gae_path, self.servlet_path,
                         self.logging_path]),
        exclude)


if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
"""Ordered set implementations."""

import collections


class _OrderedSetBase(object):

  def __init__(self, iterable=None):
    self._set = collections.OrderedDict()
    if iterable:
      for i in iterable:
        self._set[i] = 1

  def __len__(self):
    return len(self._set)

  def __contains__(self, thing):
    return thing in self._set

  def __iter__(self):
    return iter(self._set)


class _MutableSetBase(_OrderedSetBase):
  def add(self, thing):  # pylint:disable=g-bad-name
    self._set[thing] = 1

  def discard(self, thing):  # pylint:disable=g-bad-name
    if thing in self._set:
      del self._set[thing]

  def clear(self):  # pylint:disable=g-bad-name
    self._set.clear()


class FrozenOrderedSet(_OrderedSetBase, collections.Set, collections.Hashable):
  __hash__ = collections.Set._hash


class MutableOrderedSet(_MutableSetBase, collections.MutableSet):
  pass



#!/usr/bin/python2.7
# Copyright 2010 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for data_value.py."""


from google.apputils import basetest

from googleapis.codegen import data_types
from googleapis.codegen import data_value
from googleapis.codegen import language_model
from googleapis.codegen import schema
from django import template as django_template  # pylint: disable=g-bad-import-order


class DataValueTest(basetest.TestCase):

  def setUp(self):
    super(DataValueTest, self).setUp()
    self.language_model = language_model.LanguageModel(class_name_delimiter='|')

  def testDataValue(self):
    foo_def_dict = {
        'className': 'Foo',
        'type': 'string',
        }
    prototype = data_types.DataType(
        foo_def_dict, None, language_model=self.language_model)
    dv = data_value.DataValue(3, prototype)

    # Basic Checks
    self.assertEqual(3, dv.value)
    self.assertEqual(prototype, dv.data_type)
    self.assertEqual({}, dv.metadata)
    self.assertEqual('Foo', dv.code_type)

    dv.metadata['foo'] = 'bar'
    self.assertEqual({'foo': 'bar'}, dv.metadata)

    dv.SetValue('four')
    self.assertEqual(dv.value, 'four')

    self.assertEqual(self.language_model, dv.GetLanguageModel())
    other_language_model = language_model.LanguageModel(
        class_name_delimiter='+')
    dv.SetLanguageModel(other_language_model)
    self.assertEqual(other_language_model, dv.GetLanguageModel())

    # Now that we've set a local language model... make sure the codepath
    # for setting the data_type's language model gets exercised.
    self.assertEqual('Foo', dv.code_type)

    # Check that the constructor doesn't freak if an odd object is passed in
    dv = data_value.DataValue(object, prototype)
    self.assertEqual(dv.value, object)

    # A standard case is the prototype being a Property object.  It is not
    # uncommon that the Property's data_type is a SchemaReference. To verify
    # this case is handled correctly we must fake an API.
    bar_def_dict = {
        'className': 'Foo',
        'type': 'string',
        }

    class MockApi(object):

      def __init__(self):
        self.model_module = None

      def SetSchema(self, s):
        self.schema = s

      def SetSchemaRef(self, schema_ref):
        self.schema_ref = schema_ref

      # pylint: disable=unused-argument
      def ToClassName(self, name, element, element_type):
        return name

      # pylint: disable=unused-argument
      def SchemaByName(self, schema_name):
        return self.schema

      # pylint: disable=unused-argument
      def DataTypeFromJson(self, unused_def_dict, tentative_class_name,
                           parent=None, wire_name=None):
        return self.schema_ref

      def NestedClassNameForProperty(self, name, owning_schema):
        return '%s%s' % (owning_schema.class_name, name)

    mock_api = MockApi()
    bar_schema = schema.Schema(mock_api, 'Bar', bar_def_dict)
    mock_api.SetSchema(bar_schema)
    schema_ref = data_types.SchemaReference('Bar', mock_api)
    mock_api.SetSchemaRef(schema_ref)

    prototype = schema.Property(mock_api, schema_ref, 'baz', foo_def_dict)
    dv = data_value.DataValue('3', prototype)
    # Assure all the unwrapping gymnastics in the DataValue constructor did
    # their job correctly.
    self.assertEqual(mock_api.schema, dv.data_type)


class DataValueRenderingTest(basetest.TestCase):
  """Tests for DataValue rendering methods in template_helpers."""

  def _GetContext(self, data=None):
    return django_template.Context(data or {})

  def testDataContextNode(self):
    # This happens to test the "value_of" tag as well.
    lang_model = language_model.LanguageModel('|')
    foo_def_dict = {
        'className': 'Foo',
        'type': 'string',
        }
    prototype = data_types.DataType(
        foo_def_dict, None, language_model=lang_model)
    dv = data_value.DataValue('four', prototype)

    source = '{% value_of data %}'
    template = django_template.Template(source)

    context = self._GetContext({'data': dv})
    self.assertEquals('"four"', template.render(context))

    context = self._GetContext({'data': 'foo'})
    self.assertRaises(
        django_template.TemplateSyntaxError, template.render, context)

    bar_def_dict = {
        'className': 'Foo',
        'type': 'parrot',
        }
    prototype = data_types.DataType(
        bar_def_dict, None, language_model=lang_model)
    dv = data_value.DataValue('fred', prototype)

    context = self._GetContext({'data': dv})
    self.assertRaises(
        django_template.TemplateSyntaxError, template.render, context)


if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
#
# Copyright 2012 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for tar_library_package."""

__author__ = 'sammccall@google.com (Sam McCall)'

from io import BytesIO
import os
import tarfile

import gflags as flags
from google.apputils import basetest
from googleapis.codegen.filesys import tar_library_package

FLAGS = flags.FLAGS


class TarLibraryPackageTest(basetest.TestCase):
  _FILE_NAME = 'a_test'
  _DISALLOWED_FILE_NAME = 'unicode_☃☄'
  _FILE_CONTENTS = u'this is a test - ☃☄'
  _TEST_DATA_DIR = os.path.join(os.path.dirname(__file__), 'testdata')

  def setUp(self):
    self._output_stream = BytesIO()
    self._package = tar_library_package.TarLibraryPackage(
        self._output_stream)

  def tearDown(self):
    pass

  def testAsciiFilenames(self):
    self.assertRaises(UnicodeError, self._package.StartFile,
                      self._DISALLOWED_FILE_NAME)

  def testBasicWriteFile(self):
    stream = self._package.StartFile(self._FILE_NAME)
    stream.write(self._FILE_CONTENTS)
    self._package.EndFile()
    self._package.DoneWritingArchive()

    # read it back and verify
    archive = tarfile.open(fileobj=BytesIO(self._output_stream.getvalue()),
                           mode='r:gz')
    info_list = archive.getmembers()
    self.assertEquals(1, len(info_list))
    self.assertEquals(self._FILE_NAME, info_list[0].name)
    self.assertEquals(len(self._FILE_CONTENTS.encode('utf-8')),
                      info_list[0].size)

  def testBasicWriteFileUncompressed(self):
    output_stream = BytesIO()
    package = tar_library_package.TarLibraryPackage(
        output_stream, compress=False)
    stream = package.StartFile(self._FILE_NAME)
    stream.write(self._FILE_CONTENTS)
    package.EndFile()
    package.DoneWritingArchive()

    # read it back and verify
    archive = tarfile.open(fileobj=BytesIO(output_stream.getvalue()), mode='r')
    info_list = archive.getmembers()
    self.assertEquals(1, len(info_list))
    self.assertEquals(self._FILE_NAME, info_list[0].name)
    self.assertEquals(len(self._FILE_CONTENTS.encode('utf-8')),
                      info_list[0].size)

  def testStartAutomaticallyClosesPreviousFile(self):
    stream = self._package.StartFile(self._FILE_NAME)
    stream.write(self._FILE_CONTENTS)
    file_name_2 = '%s_2' % self._FILE_NAME
    stream = self._package.StartFile(file_name_2)
    stream.write(self._FILE_CONTENTS)
    self._package.EndFile()
    self._package.DoneWritingArchive()
    # read it back and verify
    archive = tarfile.open(fileobj=BytesIO(self._output_stream.getvalue()),
                           mode='r:gz')
    info_list = archive.getmembers()
    self.assertEquals(2, len(info_list))
    self.assertEquals(self._FILE_NAME, info_list[0].name)
    self.assertEquals(file_name_2, info_list[1].name)

  def testDoneAutomaticallyEndsFile(self):
    stream = self._package.StartFile(self._FILE_NAME)
    stream.write(self._FILE_CONTENTS)
    self._package.DoneWritingArchive()

    # read it back and verify
    archive = tarfile.open(fileobj=BytesIO(self._output_stream.getvalue()),
                           mode='r:gz')
    info_list = archive.getmembers()
    self.assertEquals(1, len(info_list))
    self.assertEquals(self._FILE_NAME, info_list[0].name)

  def testIncludeFile(self):
    made_up_dir = 'new_directory/'
    made_up_path = '%sfile1.txt' % made_up_dir
    # testdata/file1.txt is 125 bytes long.
    expected_size = 125
    self._package.IncludeFile(os.path.join(self._TEST_DATA_DIR, 'file1.txt'),
                              made_up_path)
    self._package.DoneWritingArchive()

    # read it back and verify
    archive = tarfile.open(fileobj=BytesIO(self._output_stream.getvalue()),
                           mode='r:gz')
    info_list = archive.getmembers()
    self.assertEquals(1, len(info_list))  # no explicit folders
    self.assertEquals(made_up_path, info_list[0].name)
    self.assertEquals(expected_size, info_list[0].size)

  def testManyFiles(self):
    top_of_tree = os.path.join(self._TEST_DATA_DIR, 'tree/')
    total_files_in_testdata_tree = 3  # determined by hand
    paths = []
    for root, unused_dirs, file_names in os.walk(top_of_tree):
      for file_name in file_names:
        paths.append(os.path.join(root, file_name))
    self._package.IncludeManyFiles(paths, top_of_tree)
    self._package.DoneWritingArchive()

    # check it
    archive = tarfile.open(fileobj=BytesIO(self._output_stream.getvalue()),
                           mode='r:gz')
    info_list = archive.getmembers()
    self.assertEquals(total_files_in_testdata_tree, len(info_list))

  def testManyFilesError(self):
    files = [os.path.join(self._TEST_DATA_DIR, file_name)
             for file_name in ['tree/abc', 'tree/def', 'file1.txt']]
    self.assertRaises(ValueError,
                      self._package.IncludeManyFiles,
                      files,
                      os.path.join(self._TEST_DATA_DIR, 'tree/'))

  def testFileProperties(self):
    self.assertEquals('tgz', self._package.FileExtension())
    self.assertEquals('application/x-gtar-compressed', self._package.MimeType())
    uncompressed = tar_library_package.TarLibraryPackage(
        BytesIO(), compress=False)
    self.assertEquals('tar', uncompressed.FileExtension())
    self.assertEquals('application/x-gtar', uncompressed.MimeType())


if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
"""Tests for ordered_set.py."""

from google.apputils import basetest

from googleapis.codegen.utilities import ordered_set


class OrderedSetTest(basetest.TestCase):

  def testFrozenSet(self):
    s = ordered_set.FrozenOrderedSet(range(5))
    self.assertEquals(5, len(s))
    self.assertTrue(4 in s)
    self.assertEquals(range(5), list(s))

  def testMutableSet(self):
    s = ordered_set.MutableOrderedSet()
    for i in range(5):
      s.add(i)
    self.assertEquals(5, len(s))
    self.assertTrue(4 in s)
    self.assertEquals(range(5), list(s))

    s.remove(3)
    self.assertEquals([0, 1, 2, 4], list(s))
    s.clear()
    self.assertEquals(0, len(s))


if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
# Copyright 2012 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""A LibraryPackage that creates a .tar.gz file.

This module aids in the construction of a .tar.gz file containing all the
components generated and required by a library.
"""

__author__ = 'sammccall@google.com (Sam McCall)'

from io import BytesIO
import StringIO
import tarfile
import time

from googleapis.codegen.filesys.library_package import LibraryPackage


class TarLibraryPackage(LibraryPackage):
  """The library package."""

  def __init__(self, stream, compress=True):
    """Create a new TarLibraryPackage.

    Args:
      stream: (file) A file-like object to write to.
      compress: (boolean) Whether to gzip-compress the output.
    """
    super(TarLibraryPackage, self).__init__()
    mode = 'w:gz' if compress else 'w'
    self._tar = tarfile.open(fileobj=stream, mode=mode)
    self._current_file_data = None
    self._compress = compress

  def StartFile(self, name):
    """Start writing a named file to the package.

    Args:
      name: (str) path which will identify the contents in the archive.

    Returns:
      A file-like object to write the contents to.
    """
    self.EndFile()
    self._current_file_data = StringIO.StringIO()
    name = '%s%s' % (self._file_path_prefix, name)
    # Let this explode if the name is not ascii.
    self._current_file_name = name.encode('ascii')
    return self._current_file_data

  def EndFile(self):
    """Flush the current output file to the tar container."""
    if self._current_file_data:
      info = tarfile.TarInfo(self._current_file_name)
      info.mtime = time.time()
      info.mode = 0644
      data = self._current_file_data.getvalue()
      if isinstance(data, unicode):
        data = data.encode('utf-8')
      info.size = len(data)
      self._tar.addfile(info, BytesIO(data))
      self._current_file_data.close()
      self._current_file_data = None

  def DoneWritingArchive(self):
    """Signal that we are done writing the entire package.

    This method must be called to flush the tar file directory to the output
    stream.
    """
    if self._tar:
      self.EndFile()
      self._tar.close()
      self._tar = None

  def FileExtension(self):
    """Returns the file extension for this archive, either tar or tgz."""
    if self._compress:
      return 'tgz'
    else:
      return 'tar'

  def MimeType(self):
    """Returns the MIME type for this archive."""
    if self._compress:
      return 'application/x-gtar-compressed'
    return 'application/x-gtar'



#!/usr/bin/python2.7
# Copyright 2014 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Objective-C library generator's import manager."""

__author__ = 'qwandor@google.com (Andrew Walbran)'


class ObjCImportManager(object):
  """The import manager for the Objective-C code generator."""

  def __init__(self, element):
    """Construct an import manager for the specified element.

    Args:
      element: (Schema) or (Api). The element we want to create an import
        manager for.
    """
    self._element = element
    self._element.SetTemplateValue('importManager', self)
    self._type_dependencies = set()
    self._imports = set()

  @classmethod
  def ForElement(cls, element):
    manager = element.get('importManager')
    if manager:
      return manager
    # instantiation installs manager inside element.
    return cls(element)

  def ImportLists(self):
    """Returns the set of import lists."""
    return [sorted(self._imports)]

  def AddDataType(self, data_type):
    self._type_dependencies.add(data_type.code_type)
    return self.AddImport('"%s.h"' % data_type.code_type)

  def AddImport(self, filename):
    """Adds the specified import to the import manager.

    Args:
      filename: (str) The filename we want to add to this instance of import
        manager.

    Returns:
      True- If adding the import is successful.
      False- If adding the import fails.
    """
    self._imports.add(filename)
    return True

  @property
  def type_dependencies(self):
    return sorted(self._type_dependencies)



#!/usr/bin/python2.7
"""A shim to surface codegen scripts as setuptools console scripts."""

from google.apputils import run_script_module


def RunGenerateLibrary():
  # pylint:disable=g-import-not-at-top
  from googleapis.codegen import generate_library
  run_script_module.RunScriptModule(generate_library)


def RunExpandTemplates():
  # pylint:disable=g-import-not-at-top
  from googleapis.codegen import expand_templates
  run_script_module.RunScriptModule(expand_templates)



#!/usr/bin/python2.7
"""Add comments to Json configuration files."""

import json
import re
import sys


# Note that we use $, not \n, because we want to retain the newlines of the
# original, so that line numbers returned by json parse errors can be easily
# mapped to the source.  For similar reasons we don't match whitespace with
# '\s', which would match '\n' and discard newlines before a comment.
COMMENT_PAT = re.compile(r'^[ \t]*#.*$', re.MULTILINE)


def _StripComments(json_string):
  """Strip comments from a json-with-comments string.

  Any line beginning with a pound sign, or with whitespace followed by a pound
  sign, is removed.  Comments are not allowed on the same line as json
  constructs.

  Args:
    json_string: (str) A json string which may contain comments.
  Returns:
    A string without comments.
  """
  return COMMENT_PAT.sub('', json_string)


def Load(fp, **kw):
  """Load json with comments from a file.

  Args:
    fp: (file) A fileish object.
    **kw: (dict) Keyword arguments to pass to the underlying json parser.
  Returns:
    Decoded json data.
  """
  raw = fp.read()
  return Loads(raw, **kw)


def Loads(json_string, **kw):
  """Load json with comments from a string.

  Args:
    json_string: (str|unicode) A string.
    **kw: (dict) Keyword arguments to pass to the underlying json parser.
  Returns:
    Decoded json data.
  """
  stripped = _StripComments(json_string)
  return json.loads(stripped, **kw)


if __name__ == '__main__':
  if len(sys.argv) > 1:
    json_in = open(sys.argv[1])
  else:
    json_in = sys.stdin
  data = Load(json_in)
  json.dump(data, sys.stdout, indent=2)



#!/usr/bin/python2.7


import errno
import os



class FileDoesNotExist(Exception):
  """File does not exist."""


def GetFileContents(filename):
  """Returns the contents of a file.


  Args:
    filename: path to a file.
  Returns:
    a string.
  Raises:
    FileDoesNotExist: if the file does not exist
    IOError: for other local IO errors
  """
  try:
    return open(filename).read()
  except IOError as e:
    if e.errno == errno.ENOENT:
      raise FileDoesNotExist(filename)
    raise


def IsFile(filename):
  """Returns whether the named file is a regular file.

  Args:
    filename: path to a file.
  Returns:
    bool: whether the file is a regular file.
  """
  return os.path.isfile(filename)


def IterFiles(directory):
  """yield all files beneath a directory."""
  for root, unused_dirs, filenames in os.walk(directory):
    for f in filenames:
      yield os.path.join(root, f)


